# R&D Policy

Replication materials for:

"Optimal Taxation and R&D Policies" by Ufuk Akcigit, Douglas Hanley, and Stefanie Stantcheva, *Econometrica*, 2021

## Setup

To install, if you don't already have this repository locally, simply clone it with
```bash
git clone https://github.com/iamlemec/rdpolicy
```
All non-standard package requirements are listed in `requirements.txt`. You can automatically install these with `pip` by running
```bash
pip install -r requirements.txt
```
The core equilibrium and optimal policy code can be found in `rdpolicy.py`. Most other python scripts ultimately call functions in this file. To test that things are working, you can compute the optimal non-linear policy by running
```python
import rdpolicy as rp
econ = rp.Economy()
eq, ts = econ.optimal()
```
The variable `eq` should contain the full state-contingent description, and `ts` should be a DataFrame with the resulting time series aggregates.

## Results

The baseline specification uses parameters from `params/estimated_allfirms.toml` that are estimated using data moments from `data/moments_allfirms.toml`. To generate figures for the baseline specification and output them to the `images` folder, simply run
```bash
python3 paper_graph.py
```
This command also takes a number of options that you can see by passing the `--help` argument, for instance, you can specify which parameter set to use with `--params` and which output prefix to use with `--prefix`. This command also generates associated latex figures in the `latex` directory. The following will generate all the results in the main paper (and some extras)
```bash
python3 paper_graph.py --params params/estimated_allfirms.toml --prefix baseline
python3 paper_graph.py --params params/rho80.toml --prefix rho80
python3 paper_graph.py --params params/persist90.toml --prefix persist90
python3 paper_graph.py --params params/zeta0.toml --prefix zeta0
```

To output tables for the baseline specification to `latex/tables.tex` run
```bash
python3 paper_table.py
```
You can also specify an alternative output path with the `--output` flag.

## Estimation

You can estimate model parameters using `estimation.py`. Without any arguments, this will run the baseline estimation using 5 threads. You can modify the number of threads used with the `--threads` flag. The estimation might take about an hour, so be patient!

## Optimal Policy

Computing optimal constrained policies for the variety of functional forms considered can be done using `optim_policy.py`. You need to specify the type of policy considered (such as `lin`) using `--ptype`. By default this will use simple policies from `policy/default_policies.toml`. To start from computed optimal policies use `--pfile policy/optimal.toml`.
