import toml
import argparse
import rdpolicy as rp

parser = argparse.ArgumentParser(description='Find optimal policies using parallel annealing.')
parser.add_argument('--params', type=str, help='parameter values to use')
parser.add_argument('--ptag', type=str, default='baseline', help='tag to identify params by')
parser.add_argument('--control', action='store_true', help='control production (rather than monopoly)')
args = parser.parse_args()

# solver options
sol_args = {
    'tol'    : 1e-13,
    'output' : True,
    'bail'   : True,
    'reps'   : 1000,
}

# generate tag
stag = 'control' if args.control else 'monopoly'
tag = f'{args.ptag}_{stag}'

# calculate optimum
econ = rp.Economy(params=args.params)
opt, _ = econ.optimal(control=args.control, **sol_args)
eq, _ = econ.equilibrium(control=args.control, **sol_args)

# print final results
print(toml.dumps({
    'opt': {tag: {'welfare': opt.welfare}},
    'eq': {tag: {'welfare': eq.welfare}}
}))
