import os
import toml
import argparse
import numpy as np
from itertools import product

parser = argparse.ArgumentParser(description='Generate bootstrap targets.')
parser.add_argument('--target', type=str, default=None, help='data moments to target')
parser.add_argument('--stderr', type=str, default=None, help='variances of targets')
parser.add_argument('--output', type=str, default='bootstrap/data', help='output directory')
parser.add_argument('--nodes', type=int, default=10, help='number of processing nodes')
parser.add_argument('--draws', type=int, default=10, help='number of draws per node')
parser.add_argument('--seed', type=int, default=None, help='seed to use for generation')
args = parser.parse_args()

with open(args.target) as fid:
    moment = toml.load(fid)
with open(args.stderr) as fid:
    stderr = toml.load(fid)
n_mmt = len(moment)

mmt_vec = np.array([moment[m] for m in moment])
std_vec = np.array([stderr[m] for m in moment])

rs = np.random.RandomState(args.seed) if args.seed is not None else np.random
rfact = rs.randn(args.nodes, args.draws, n_mmt)
mmt_new = mmt_vec[None,None,:] + std_vec[None,None,:]*rfact

for i in range(args.nodes):
    node_out = f'{args.output}/{i}'
    if not os.path.exists(node_out):
        os.makedirs(node_out)
    for j in range(args.draws):
        draw_out = f'{node_out}/moments_{j}.toml'
        mmt_out = {k: v for k, v in zip(moment, mmt_new[i,j,:])}
        with open(draw_out, 'w+') as fout:
            toml.dump(mmt_out, fout)
